package com.bmw.cascade.pruefstand.ediabas;

/** Diese Exception wird bei einem Fehler im Ediabas abgefeuert.
 * @author Volker Funke, BRAINBOW EDV-Beratung GmbH
 * @version 1.0
 */
public class ApiCallFailedException extends Exception {
  int ediabasErrorCode;

  String ediabasErrorMessage;

  /** Erzeugt eine neue ApiCallFailedException.
   */
  public ApiCallFailedException() {
    super();
  }

  /** Erzeugt eine neue ApiCallFailedException mit einer Meldung.
   * @param message eine Nachricht ber die Natur des Fehlers.
   */
  public ApiCallFailedException(String message) {
    super( message );
  }

  /** Erzeugt eine neue ApiCallFailedException mit einer Meldung und dem Ediabas-Fehlercode sowie der Ediabas-Fehlermeldung.
   * <B>Zur Zeit wird dieses Feature im Ediabas nicht benutzt.</B>
   * @param message Ein beschreibender Text.
   * @param ediabasErrorCode der Ediabas-Fehlercode
   * @param ediabasErrorMessage die Ediabas-Fehlermeldung.
   */
  public ApiCallFailedException(String message,int ediabasErrorCode,String ediabasErrorMessage) {
    super( message );
    this.ediabasErrorCode = ediabasErrorCode;
    this.ediabasErrorMessage = ediabasErrorMessage;
  }

  /** Liefert den Ediabas-Fehlercode, sofern vorhanden.
   * @return wenn vorhanden, der Fehlercode, sonst 0.
   */
  public int getErrorCode() {
    return ediabasErrorCode;
  }

  /** Liefert den Ediabas-Fehlertext, sofern vorhanden.
   * @return der Ediabas-Fehlertext, sofern vorhanden, sonst null.
   */
  public String getErrorMessage() {
    return ediabasErrorMessage;
  }


}