package com.bmw.cascade.pruefstand.ediabas;

/** Diese Klasse enthlt alle in Java untersttzten Ediabas-API-Calls als statische
 * Methoden. Alle im Ediabas vorhandenen Datentypen werden, sofern mglich, in Java-Datentypen
 * umgewandelt.
 * <BR>
 * Die bentigte Ediabas-Version ist <B>Version 6.0</B>
 * @author Volker Funke, BRAINBOW EDV-Beratung GmbH
 * @version 1.0
 */
public class Ediabas {
    static{
        System.loadLibrary( "apijav32" );
    }
    
    /** Konstante fr fr das Ediabas APIFORMAT_CHAR.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_CHAR =0;
    
    /** Konstante fr fr das Ediabas APIFORMAT_BYTE.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_BYTE =1;
    
    /** Konstante fr fr das Ediabas APIFORMAT_INTEGER.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_INTEGER =2;
    
    /** Konstante fr fr das Ediabas APIFORMAT_WORD.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_WORD =3;
    
    /** Konstante fr fr das Ediabas APIFORMAT_LONG.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_LONG =4;
    
    /** Konstante fr fr das Ediabas APIFORMAT_DWORD.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_DWORD =5;
    
    /** Konstante fr fr das Ediabas APIFORMAT_TEXT.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_TEXT =6;
    
    /** Konstante fr fr das Ediabas APIFORMAT_BINARY.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_BINARY =7;
    
    /** Konstante fr fr das Ediabas APIFORMAT_REAL.
     * @see #apiResultFormat
     */
    public static final int APIFORMAT_REAL =8;
    
    /** Einer der Rckgabewerte von apiState.
     * @see #apiState
     */
    public static final int APISTATE_BUSY =0;
    
    /** Einer der Rckgabewerte von apiState.
     * @see #apiState
     */
    public static final int APISTATE_READY =1;
    
    /** Einer der Rckgabewerte von apiState.
     * @see #apiState
     */
    public static final int APISTATE_BREAK =2;
    
    /** Einer der Rckgabewerte von apiState.
     * @see #apiState
     */
    public static final int APISTATE_ERROR =3;
    
    /** Fhrt den Ediabas-Call <CODE>apiInit()</CODE> aus.
     * @throws ApiCallFailedException bei einem Fehler whrend des Aufrufs.
     */
    public static native void apiInit() throws ApiCallFailedException;
    
    /** Fhrt den Ediabas-Call <CODE>apiEnd()</CODE> aus.
     */
    public static native void apiEnd();
    
    /** Ruft die Ediabas-Funktion <CODE>apiJob()</CODE>
     * <BR>Nomenklatur der Original C-Funktion:<BR>
     * <CODE>void apiJob(const char *,const char *,const char *,const char *);</CODE>
     * <BR>
     * @param sgGruppeVariante Gruppendateiname
     * @param job Jobname
     * @param parameter Parameter wie im Ediabas verlangt.
     * @param ergebnisse Gewnschte Ergebnisse, wie im Ediabas verlangt.
     */
    public static native void apiJob(String sgGruppeVariante,String job,String parameter,String ergebnisse);
    
    /** Ruft den Ediabas-Call <CODE>apiJobData()</CODE>auf.
     * <BR>Nomenklatur der Original C-Funktion:<BR>
     * <CODE>C: void apiJobData(const char *ecu,const char *job, const unsigned char *parabuf,int paralen,const char *result)</CODE>
     * @param sgGruppeVariante Steuergertedateiname
     * @param job Jobname
     * @param buffer Puffer mit den Binrdaten, jedes Element des Puffers darf nur Werte zwischen 0 und 255 enthalten.
     * @param result die gewnschten Ergebnisse
     * @throws ApiCallFailedException bei einem Ediabasfehler.
     */
    public static native void apiJobData(String sgGruppeVariante,String job,short[] buffer,String result) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultByte auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultByte(APIBYTE *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnisbyte, ein Wert zwischen 0 und 255.
     */
    public static native short apiResultByte(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultBinary auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultBinary(APIBINARY *,APIWORD *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis, als Array. Jedes Element hat einen Wert zwischen 0 und 255.
     */
    public static native short[] apiResultBinary(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultChar auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultChar(APICHAR *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native char apiResultChar(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultDWord auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultDWord(APIDWORD *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis. Es ist ein long, das das APIDWORD ein unsigned 32-Bit Wert ist.
     */
    public static native long apiResultDWord(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Liefert den Datentyp des Ergebnisses. Ruft den EdiabasCall <CODE>apiResultFormat</CODE> auf<BR>
     * Nomenklatur der Original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultFormat(APIRESULTFORMAT *,const char *,APIWORD)</CODE>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer.
     * @throws ApiCallFailedException Bei einem EdiabasFehler.
     * @return den Datentyp.
     * @see #APIFORMAT_CHAR
     * @see #APIFORMAT_INTEGER
     * @see #APIFORMAT_WORD
     * @see #APIFORMAT_LONG
     * @see #APIFORMAT_DWORD
     * @see #APIFORMAT_TEXT
     * @see #APIFORMAT_BINARY
     * @see #APIFORMAT_REAL
     */
    public static native int apiResultFormat(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultInt auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultInt(APIINTEGER *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native short apiResultInt(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultLong auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultLong(APILONG *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native int apiResultLong(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultName auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultName(char *,APIWORD,APIWORD)</CODE><BR>
     * @param resultNummer Nummer des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native String apiResultName(int resultNummer,int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultNumber auf. Liefert die Anzahl der Ergebnisse.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultNumber(APIWORD *,APIWORD)</CODE><BR>
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return die Anzahl.
     */
    public static native int apiResultNumber(int satzNummer) throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultReal auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultReal(APIREAL *,const char *,APIWORD)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native double apiResultReal(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** Liefert die Anzahl der Ergebnisstze im Ediabas. Ruft den Ediabas-Call <CODE>apiResultSets()</CODE> auf.
     * <BR>Der originale C-Code:<BR> <CODE>APIBOOL apiResultSets(APIWORD *)</CODE>
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return die Anzahl der Ergebnisstze.
     */
    public static native int apiResultSets() throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultText auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE> APIBOOL apiResultText(APITEXT *,const char *,APIWORD, const char *)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @param format Format des Ergebnisses, gem der Ediabas-Spezifikation
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native String apiResultText(String resultName,int satzNummer,String format) throws ApiCallFailedException;
    
    /** Liefert den Namen der geladenen SGBD.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultVar(APITEXT *ecu)</CODE><BR>
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return der Name des SGBD.
     */
    public static native String apiResultVar() throws ApiCallFailedException;
    
    /** Ruft den EdiabasCall apiResultWord auf.
     * <BR>Nomenklatur der original C-Funktion:<BR>
     * <CODE>APIBOOL apiResultWord(APIWORD *buf,const char *result,APIWORD set)</CODE><BR>
     * @param resultName Name des Ergebnisses.
     * @param satzNummer Ergebnissatznummer
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das Ergebnis.
     */
    public static native int apiResultWord(String resultName,int satzNummer) throws ApiCallFailedException;
    
    /** ndert die Ediabas Konfiguration.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE>APIBOOL apiSetConfig(const char *configName,const char *configValue)</CODE>
     * @param configName Name der Konfiguration.
     * @param configValue Wert fr die Konfiguration.
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     */
    public static native void apiSetConfig(String configName,String configValue) throws ApiCallFailedException;
    
    /** Liefert den Zustand des Ediabas.
     * <BR>Die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE> int apiState(void)</CODE>
     * @return der Zustand des Ediabas
     * @see #APISTATE_BUSY
     * @see #APISTATE_READY
     * @see #APISTATE_BREAK
     * @see #APISTATE_ERROR
     */
    public static native int apiState();
    
    /** Bricht einen laufenden Ediabas-Job ab.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE>void apiBreak(void)</CODE>
     */
    public static native void apiBreak();
    
    /** Liefert den Fortschritt eines Ediabas-Jobs.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE>int apiJobInfo(char *infoText)</CODE>
     * @param infoText Infotext
     * @return der prozentuale Fortschritt von  0 bis 100.
     */
    public static native int apiJobInfo(String infoText);
    
    /** Liefert den aktuellen Ediabas-Fehlercode.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE>int apiErrorCode(void)</CODE>
     * @return der Fehlercode.
     */
    public static native int apiErrorCode();
    
    /** Liefert den aktuellen Ediabas-Fehlertext.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE> char * apiErrorText(void)</CODE>
     * @return der Fehlertext.
     */
    public static native String apiErrorText();
    
    /** Liefert die aktuelle Konfiguration.
     * <BR> die Nomenklatur der Original-C-Funktion:<BR>
     * <CODE>APIBOOL apiGetConfig(const char *configName,char *configValue)</CODE>
     * @param configName Name der Konfiguration.
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return die Konfiguration.
     */
    public static native String apiGetConfig(String configName) throws ApiCallFailedException;
    
    /** Liefert ein Ediabas-Ergebnis in aufbereiteter Form, gepackt in eine Javaklassse.
     * @throws ApiCallFailedException bei einem Ediabas-Fehler.
     * @return das aufbereitete Ergebnis.
     */
    public static EdiabasResult getResult() throws ApiCallFailedException {
        int anzahlStze = apiResultSets();
        int anzahlResults;
        String resultName;
        int resultFormat;
        EdiabasResultSet[] ergebnisStze = new EdiabasResultSet[anzahlStze+1];
        for( int i=0; i<=anzahlStze; i++ ) {
            anzahlResults = apiResultNumber( i );
            ergebnisStze[i] = new EdiabasResultSet( anzahlResults );
            for( int j=1; j<=anzahlResults; j++ ) {
                resultName = apiResultName( j, i );
                resultFormat = apiResultFormat( resultName, i );
                switch( resultFormat ) {
                    case APIFORMAT_CHAR:
                        ergebnisStze[i].put( new EdiabasResultLong( resultName, apiResultChar( resultName, i ) ) );
                        break;
                    case APIFORMAT_BYTE:
                        ergebnisStze[i].put(  new EdiabasResultLong( resultName,apiResultByte( resultName, i ) ) );
                        break;
                    case APIFORMAT_INTEGER:
                        ergebnisStze[i].put( new EdiabasResultLong( resultName, apiResultInt( resultName, i ) ) );
                        break;
                    case APIFORMAT_LONG:
                        ergebnisStze[i].put( new EdiabasResultLong( resultName, apiResultLong( resultName, i ) ) );
                        break;
                    case APIFORMAT_WORD:
                        ergebnisStze[i].put( new EdiabasResultLong( resultName, apiResultWord( resultName, i ) ) );
                        break;
                    case APIFORMAT_DWORD:
                        ergebnisStze[i].put( new EdiabasResultLong( resultName, apiResultDWord( resultName, i ) ) );
                        break;
                    case APIFORMAT_TEXT:
                        ergebnisStze[i].put( new EdiabasResultText( resultName, apiResultText( resultName, i, "" ) ) );
                        break;
                    case APIFORMAT_BINARY:
                        ergebnisStze[i].put( new EdiabasResultBinary( resultName, apiResultBinary( resultName, i ) ) );
                        break;
                    case APIFORMAT_REAL:
                        ergebnisStze[i].put( new EdiabasResultReal( resultName, apiResultReal( resultName, i ) ) );
                        break;
                    default:
                        throw new ApiCallFailedException( "Unknown API_FORMAT" );
                }
            }
        }
        return new EdiabasResult( ergebnisStze );
    }
    
    /**
     * fhrt einen Diagnose-Job aus. Hier sollte dann auch das Fehlerhandling rein ,
     * sobald dies feststeht (Retry, ...). Zur Zeit somit reines Weiterreichen.
     * @param sgbd Name der SGBD bzw. der Gruppendatei.
     * @param job Name des auszufhrenden Jobs.
     * @param par Parameter (Leerstring oder durch Semikolon getrennter ParametzerString) zum Jobs.
     * @param res Name(n) der angeforderten Results.
     * return Status des Diagnoseaufrufes, ist vom Aufrufer zu kontrollieren.
     */
    public static String executeDiagJob(String sgbd, String job, String par, String res) throws ApiCallFailedException, EdiabasResultNotFoundException {
        // MM: Bug in Ediabas - jobres darf nur 240? (wahrscheinlich 255) Zeichen lang sein
        if (res.length()>240) res="";
        apiJob(sgbd,job,par,res);
        return getDiagResultValue( apiResultSets(), "JOB_STATUS" );
    }
    
    /**
     * fhrt einen Diagnose-JobData aus. Hier sollte dann auch das Fehlerhandling rein,
     * sobald dies feststeht (Retry, ...). Zur Zeit somit reines Weiterreichen.
     * @param sgbd Name der SGBD bzw. der Gruppendatei.
     * @param job Name des auszufhrenden Jobs.
     * @param data Die Binrdaten zum Jobs.
     * @param res Name(n) der angeforderten Results.
     * return Status des Diagnoseaufrufes, ist vom Aufrufer zu kontrollieren.
     */
    public static String executeDiagJobData(String sgbd, String job, short[] data, String res) throws ApiCallFailedException, EdiabasResultNotFoundException {
        apiJobData(sgbd,job,data,res);
        return getDiagResultValue( apiResultSets(), "JOB_STATUS" );
    }
    
    /**
     * liefert die Anzahl an Ergebnisstzen.
     * return Anzahl an Ergebnisstzen
     */
    public static int getDiagJobSaetze() throws ApiCallFailedException {
        return (apiResultSets()+1);
    }
    
    /**
     * liefert den Wert eines Diagnoseergebnisses im Satz 1 als String zurck
     * @param resName angeforderter Resultname.
     * return Wert des Results als String
     */
    public static String getDiagResultValue(String resName) throws ApiCallFailedException, EdiabasResultNotFoundException {
        return getDiagResultValue(1, resName);
    }
    
    /**
     * liefert den Wert eines Diagnoseergebnisses im entsprechenden Satz als String zurck
     * @param satz Satznummer, in dem das Result gesucht wird
     * @param resName angeforderter Resultname.
     * return Wert des Results als String
     */
    public static String getDiagResultValue(int satz, String resName) throws ApiCallFailedException, EdiabasResultNotFoundException {
        switch( apiResultFormat( resName, satz ) ) {
            case APIFORMAT_CHAR:
                return ( new EdiabasResultLong( resName, apiResultChar( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_BYTE:
                return (  new EdiabasResultLong( resName,apiResultByte( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_INTEGER:
                return ( new EdiabasResultLong( resName, apiResultInt( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_LONG:
                return ( new EdiabasResultLong( resName, apiResultLong( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_WORD:
                return ( new EdiabasResultLong( resName, apiResultWord( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_DWORD:
                return ( new EdiabasResultLong( resName, apiResultDWord( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_TEXT:
                return ( new EdiabasResultText( resName, apiResultText( resName, satz, "" ) ) ).stringValue().trim();
            case APIFORMAT_BINARY:
                return ( new EdiabasResultBinary( resName, apiResultBinary( resName, satz ) ) ).stringValue().trim();
            case APIFORMAT_REAL:
                return ( new EdiabasResultReal( resName, apiResultReal( resName, satz ) ) ).stringValue().trim();
            default:
                throw new ApiCallFailedException( "Unknown API_FORMAT: "+ apiResultFormat( resName, satz ) );
        }
    }
    
    
    /** Nur zum Test.
     * @param args werden ignoriert.
     */
    public static void main(String[] args) {
        try {
            long start, end;
            for( int i=0; i<5; i++ ) {
                start = System.currentTimeMillis();
                apiInit();
                end = System.currentTimeMillis();
                System.out.println( "Time for apiInit(): " + (end-start) + " ms" );
                
                start = System.currentTimeMillis();
                apiJob( "MUST_E65", "fs_lesen_detail", "17", "f_ready_nr" );
                end = System.currentTimeMillis();
                System.out.println( "Time for apiJob(): " + (end-start) + " ms" );
                
                EdiabasResult result;
                start = System.currentTimeMillis();
                result = getResult();
                end = System.currentTimeMillis();
                System.out.println( result );
                System.out.println( "Time for getting the complete result: " + (end-start) + " ms" );
                
                start = System.currentTimeMillis();
                apiEnd();
                end = System.currentTimeMillis();
                System.out.println( "Time for apiEnd(): " + (end-start) + " ms" );
            }
        } catch( ApiCallFailedException e ) {
            System.out.println( e );
        }
    }
    
    
}
